
/**
 *	Velociraptor, A Simple Java Port Scanning API
 *	Copyright (C) 2005  Kamran Zafar
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	Contact Info:
 *	Email: xeus.man@gmail.com
 */

package xeus.net.scanner;

/**
 * @author Kamran Zafar
 *  
 */
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.ConnectException;
import java.net.NoRouteToHostException;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.List;

import xeus.net.scanner.event.ScanEvent;
import xeus.net.scanner.event.ScanListener;

public class TruePortAnalyzer implements PortScanner {

    private Socket scanSock;

    private String host;

    private int start;

    private int end;

    private List listeners;

    /**
     * 
     * @param host
     *            (IP or host name)
     * @param start
     *            (Starting from port)
     * @param end
     *            (Ending to port)
     */

    public TruePortAnalyzer(String host, int start, int end) {
        scanSock = null;
        this.host = host;
        this.start = start;
        this.end = end;
        listeners = new ArrayList();
    }

    public void scan() {

        BufferedReader in = null;
        BufferedWriter out = null;
        fireScanStartedEvent(new ScanEvent(new PortProperties(host, -1,
                PortConstants.UNKNOWN)));

        for (int i = start; i <= end; i++) {

            String line = "";
            PortProperties props = new PortProperties();
            props.setHost(host);
            props.setPort(i);

            try {
                firePortAnalysisStartedEvent(new ScanEvent(props));
                scanSock = new Socket(host, i);
                scanSock.setSoTimeout(10000);
                in = new BufferedReader(new InputStreamReader(scanSock
                        .getInputStream()));

                out = new BufferedWriter(new OutputStreamWriter(scanSock
                        .getOutputStream()));

                line = in.readLine();

                if (line != null && line.indexOf("FTP") >= 0) {
                    props.setPortType(PortConstants.FTP);
                    firePortOpenEvent(new ScanEvent(props));
                } else if (line != null && line.indexOf("HTTP") >= 0) {
                    props.setPortType(PortConstants.HTTP);
                    firePortOpenEvent(new ScanEvent(props));
                } else if (line != null && line.indexOf("SMTP") >= 0) {
                    props.setPortType(PortConstants.SMTP);
                    firePortOpenEvent(new ScanEvent(props));
                }

                in.close();
                out.close();
            } catch (NoRouteToHostException nrthe) {
                firePortFilteredEvent(new ScanEvent(props));
            } catch (ConnectException ce) {
                firePortCloseEvent(new ScanEvent(props));
            } catch (SocketTimeoutException cte) {
                try {
                    out.write("GET / HTTP/1.0 \n\n");
                    out.flush();
                    line = in.readLine();

                    if (line == null || line.trim().equals("")) {
                        out.write("HELLO\n");
                        out.flush();
                        line = in.readLine();
                    }

                    if (line != null && line.indexOf("HTTP") >= 0) {
                        props.setPortType(PortConstants.HTTP);
                        firePortOpenEvent(new ScanEvent(props));
                    } else if (line != null && line.indexOf("Whois") >= 0) {
                        props.setPortType(PortConstants.WHOIS);
                        firePortOpenEvent(new ScanEvent(props));
                    } else {
                        firePortOpenEvent(new ScanEvent(props));
                    }
                } catch (Exception e) {
                    firePortOpenEvent(new ScanEvent(props));
                }
            } catch (Exception e) {
                System.out.println("Hey what the hell happened... " + e);
            } finally {
                try {
                    if(in!=null)
                        in.close();
                    if(out!=null)
                        out.close();
                    
                    if (scanSock != null && !scanSock.isClosed())
                        scanSock.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }

            firePortAnalysisEndedEvent(new ScanEvent(props));
        }

        fireScanEndedEvent(new ScanEvent(new PortProperties(host, -1,
                PortConstants.UNKNOWN)));
    }

    public void addScanListener(ScanListener listener) {
        listeners.add(listener);
    }

    public void removeScanListener(ScanListener listener) {
        listeners.remove(listener);
    }

    protected void firePortOpenEvent(ScanEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener) listeners.get(i)).portOpen(event);
    }

    protected void firePortCloseEvent(ScanEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener) listeners.get(i)).portClose(event);
    }

    protected void firePortFilteredEvent(ScanEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener) listeners.get(i)).portFiltered(event);
    }

    protected void fireScanStartedEvent(ScanEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener) listeners.get(i)).scanStarted(event);
    }

    protected void fireScanEndedEvent(ScanEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener) listeners.get(i)).scanEnded(event);
    }

    protected void firePortAnalysisStartedEvent(ScanEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener) listeners.get(i)).portAnalysisStarted(event);
    }

    protected void firePortAnalysisEndedEvent(ScanEvent event) {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener) listeners.get(i)).portAnalysisEnded(event);
    }

    public void run() {
        scan();
    }

}