
/**
 *	Velociraptor, A Simple Java Port Scanning API
 *	Copyright (C) 2005  Kamran Zafar
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	Contact Info:
 *	Email: xeus.man@gmail.com
 */

package xeus.net.scanner;

import java.io.IOException;
import java.net.ConnectException;
import java.net.NoRouteToHostException;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.List;

import xeus.net.scanner.event.ScanEvent;
import xeus.net.scanner.event.ScanListener;


/**
 * @author Kamran Zafar
 *
 */
public class GenericPortScanner implements PortScanner{
    private Socket scanSock;

    private String host;

    private int start;

    private int end;
    
    private List listeners;

    /**
     * 
     * @param host
     *            (IP or host name)
     * @param start
     *            (Starting from port)
     * @param end
     *            (Ending to port)
     */

    public GenericPortScanner(String host, int start, int end) {
        scanSock = null;
        this.host = host;
        this.start = start;
        this.end = end;
        listeners=new ArrayList();
    }

    public void scan() {

        fireScanStartedEvent(new ScanEvent(new PortProperties(host,-1,-1)));
        
        for (int i = start; i <= end; i++) {
            try {
                firePortAnalysisStartedEvent(new ScanEvent(new PortProperties(host,i,PortConstants.UNKNOWN)));
                scanSock = new Socket(host, i);
                firePortOpenEvent(new ScanEvent(new PortProperties(host,i,PortConstants.UNKNOWN)));
            } catch (NoRouteToHostException nrthe) {
                firePortFilteredEvent(new ScanEvent(new PortProperties(host,i,PortConstants.UNKNOWN)));
            } catch (ConnectException ce) {
                firePortCloseEvent(new ScanEvent(new PortProperties(host,i,PortConstants.UNKNOWN)));
            } catch (SocketTimeoutException cte) {
                firePortFilteredEvent(new ScanEvent(new PortProperties(host,i,PortConstants.UNKNOWN)));
            } catch (Exception e) {
                System.out.println("Hey what the hell happened... " + e);
            }
            finally
            {
                try {
                    scanSock.close();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }    
            }
            
            firePortAnalysisEndedEvent(new ScanEvent(new PortProperties(host,i,PortConstants.UNKNOWN)));
        }
        
        fireScanEndedEvent(new ScanEvent(new PortProperties(host,-1,-1)));
    }
    
    public void addScanListener(ScanListener listener)
    {
        listeners.add(listener);
    }
    
    public void removeScanListener(ScanListener listener)
    {
        listeners.remove(listener);
    }
    
    protected void firePortOpenEvent(ScanEvent event)
    {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener)listeners.get(i)).portOpen(event);
    }
    
    protected void firePortCloseEvent(ScanEvent event)
    {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener)listeners.get(i)).portClose(event);
    }

    protected void firePortFilteredEvent(ScanEvent event)
    {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener)listeners.get(i)).portFiltered(event);
    }
    
    protected void fireScanStartedEvent(ScanEvent event)
    {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener)listeners.get(i)).scanStarted(event);
    }

    protected void fireScanEndedEvent(ScanEvent event)
    {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener)listeners.get(i)).scanEnded(event);
    }

    protected void firePortAnalysisStartedEvent(ScanEvent event)
    {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener)listeners.get(i)).portAnalysisStarted(event);
    }
    
    protected void firePortAnalysisEndedEvent(ScanEvent event)
    {
        for (int i = 0; i < listeners.size(); i++)
            ((ScanListener)listeners.get(i)).portAnalysisEnded(event);
    }

    public void run() {
        scan();
    }
}
